// RegistryObject.cpp : Implementation of CRegistryObject

#include "stdafx.h"
#include "RegObj.h"
#include "RegistryObject.h"

/////////////////////////////////////////////////////////////////////////////
// CRegistryObject

STDMETHODIMP CRegistryObject::CreateKey(Hives hvHive, BSTR bstrKey)
{
	// Not implemented
	ReportError(E_NOTIMPL);
	return E_NOTIMPL;
}

STDMETHODIMP CRegistryObject::OpenKey(Hives hvHive, BSTR bstrKey, VARIANT varAccess)
{
	// Note: varAccess is to hold the SAM value, such that the
	// key can be opened for reading or writing, or whatever.
	// Now, it defaults to KEY_ALL_ACCESS.
	HRESULT hr = S_OK;
	try {
		// Check input hive
		HKEY hkRoot = NULL;
		switch ( hvHive ) {
			case HKCR:
			case HKCU:
			case HKLM:
				// Valid hive
				hkRoot = (HKEY)((long)hvHive | 0x80000000);
				break;

			default:
				// Invalid hive...
				throw E_INVALIDARG;

		} // switch

		// Check key.  We'll assume a non-NULL
		// BSTR contains a key we can use.  Later
		// it still may fail, but at least we're not
		// handing the Registry a NULL key string.
		USES_CONVERSION;
		LPTSTR strKey = W2T(bstrKey);
		::SysFreeString(bstrKey);
		if ( !_tcslen(strKey) ) throw E_INVALIDARG;

		// Check the optional parameter (access rights)
		DWORD dwAccess = KEY_ALL_ACCESS;
		switch ( varAccess.vt ) {
			case VT_EMPTY:
			case VT_ERROR:
				// Accept default
				break;

			case VT_I1:
			case VT_I2:
			case VT_I4:
				switch ((long)varAccess.lVal) {
					case KeyAllAccess:
						dwAccess = KEY_ALL_ACCESS;
						break;

					case KeyRead:
						dwAccess = KEY_READ;
						break;

					case KeyWrite:
						dwAccess = KEY_WRITE;
						break;

					default:
						// They blew it...
						throw E_INVALIDARG;

				} // switch
				break;

			default:
				throw E_INVALIDARG;

		} // switch

		// Try to open the key
		long iResult = m_CKey.Open(hkRoot,strKey,dwAccess);
		if ( iResult != ERROR_SUCCESS ) throw (HRESULT)GetLastError();

		// m_CKey should now have a valid Registry
		// key handle...
	} // try
	catch(HRESULT hrErr) {
		// A COM error...
		hr = hrErr;
		ReportError(hrErr);
	} // catch
	catch(...) {
		// Some other error...
		hr = E_UNEXPECTED;
		ReportError(E_UNEXPECTED);
	} // catch

	return hr;
}

STDMETHODIMP CRegistryObject::QueryValue(BSTR bstrValueName, VARIANT *pvarValueData)
{
	HRESULT hr = S_OK;
	try {
		// Check for valid key...throws invalid
		// handle error if problem
		if ( m_CKey.m_hKey == NULL ) throw E_HANDLE;

		// Extract value name...
		USES_CONVERSION;
		LPTSTR strValueName = W2T(bstrValueName);
		::SysFreeString(bstrValueName);
		if ( !_tcslen(strValueName) ) throw E_INVALIDARG;

		// Try to query the value...in this case skip
		// CRegKey::QueryValue() because we want to know
		// the type so we can complete the variant...
		DWORD dwType = REG_SZ, dwSize = _MAX_PATH;
		BYTE bData[_MAX_PATH] = {0};
		long iResult = RegQueryValueEx(m_CKey.m_hKey,
										strValueName,
										NULL,
										&dwType,
										bData,
										&dwSize);
		if ( iResult != ERROR_SUCCESS ) throw (HRESULT)GetLastError();

		// We'll support integer and string key values...
		CComVariant varReturn;
		switch ( dwType ) {
			case REG_SZ:
				{ // scope
					varReturn.vt = VT_BSTR;
					USES_CONVERSION;
					LPOLESTR ostrData = A2W((const char *)bData);
					varReturn.bstrVal = ::SysAllocString(ostrData);
				} // scope
				break;

			case REG_DWORD:
				{ // scope
					varReturn.vt = VT_I4;
					varReturn.lVal = (long)*((long*)bData);
				} // scope
				break;

			default:
				varReturn.vt = VT_ERROR;
				break;

		} // switch

		// Return variant...
		::VariantCopy(pvarValueData,&varReturn);
	} // try
	catch(HRESULT hrErr) {
		// A COM error...
		hr = hrErr;
		ReportError(hrErr);
	} // catch
	catch(...) {
		// Some other error...
		hr = E_UNEXPECTED;
		ReportError(E_UNEXPECTED);
	} // catch

	return hr;
}

STDMETHODIMP CRegistryObject::SetValue(BSTR bstrValueName, VARIANT varValueData)
{
	HRESULT hr = S_OK;
	try {
		// Check for valid key...throws invalid
		// handle error if problem
		if ( m_CKey.m_hKey == NULL ) throw E_HANDLE;

		// Extract value name...
		USES_CONVERSION;
		LPTSTR strValueName = W2T(bstrValueName);
		::SysFreeString(bstrValueName);
		if ( !_tcslen(strValueName) ) throw E_INVALIDARG;

		// Try to set the value...
		long iResult = ERROR_SUCCESS;
		switch ( varValueData.vt ) {
			case VT_BSTR:
				{ // scope
					USES_CONVERSION;
					LPTSTR strData = W2T(varValueData.bstrVal);
					iResult = m_CKey.SetValue(strData,strValueName);
				} // scope
				break;

			case VT_I1:
			case VT_I2:
			case VT_I4:
				iResult = m_CKey.SetValue((long)varValueData.lVal,strValueName);
				break;

			default:
				throw E_INVALIDARG;

		} // switch

		if ( iResult != ERROR_SUCCESS ) throw (HRESULT)GetLastError();
	} // try
	catch(HRESULT hrErr) {
		// A COM error...
		hr = hrErr;
		ReportError(hrErr);
	} // catch
	catch(...) {
		// Some other error...
		hr = E_UNEXPECTED;
		ReportError(E_UNEXPECTED);
	} // catch

	return hr;
}

STDMETHODIMP CRegistryObject::DeleteSubkey(BSTR bstrSubKey, VARIANT_BOOL bDeleteChildKeys)
{
	// Not implemented
	ReportError(E_NOTIMPL);
	return E_NOTIMPL;
}

STDMETHODIMP CRegistryObject::DeleteValue(BSTR bstrValueName)
{
	// Not implemented
	ReportError(E_NOTIMPL);
	return E_NOTIMPL;
}
