// RegistryObject.h : Declaration of the CRegistryObject

#ifndef __REGISTRYOBJECT_H_
#define __REGISTRYOBJECT_H_

#include "resource.h"       // main symbols
#include <atlctl.h>


/////////////////////////////////////////////////////////////////////////////
// CRegistryObject
class ATL_NO_VTABLE CRegistryObject : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public IDispatchImpl<IAccessRegistry, &IID_IAccessRegistry, &LIBID_REGOBJLib>,
	public CComControl<CRegistryObject>,
	public IPersistStreamInitImpl<CRegistryObject>,
	public IOleControlImpl<CRegistryObject>,
	public IOleObjectImpl<CRegistryObject>,
	public IOleInPlaceActiveObjectImpl<CRegistryObject>,
	public IViewObjectExImpl<CRegistryObject>,
	public IOleInPlaceObjectWindowlessImpl<CRegistryObject>,
	public ISupportErrorInfo,
	public IPersistStorageImpl<CRegistryObject>,
	public ISpecifyPropertyPagesImpl<CRegistryObject>,
	public IQuickActivateImpl<CRegistryObject>,
	public IDataObjectImpl<CRegistryObject>,
	public IProvideClassInfo2Impl<&CLSID_RegistryObject, NULL, &LIBID_REGOBJLib>,
	public CComCoClass<CRegistryObject, &CLSID_RegistryObject>
{
public:
	CRegistryObject()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_REGISTRYOBJECT)

DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CRegistryObject)
	COM_INTERFACE_ENTRY(IAccessRegistry)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(IViewObjectEx)
	COM_INTERFACE_ENTRY(IViewObject2)
	COM_INTERFACE_ENTRY(IViewObject)
	COM_INTERFACE_ENTRY(IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceObject)
	COM_INTERFACE_ENTRY2(IOleWindow, IOleInPlaceObjectWindowless)
	COM_INTERFACE_ENTRY(IOleInPlaceActiveObject)
	COM_INTERFACE_ENTRY(IOleControl)
	COM_INTERFACE_ENTRY(IOleObject)
	COM_INTERFACE_ENTRY(IPersistStreamInit)
	COM_INTERFACE_ENTRY2(IPersist, IPersistStreamInit)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY(ISpecifyPropertyPages)
	COM_INTERFACE_ENTRY(IQuickActivate)
	COM_INTERFACE_ENTRY(IPersistStorage)
	COM_INTERFACE_ENTRY(IDataObject)
	COM_INTERFACE_ENTRY(IProvideClassInfo)
	COM_INTERFACE_ENTRY(IProvideClassInfo2)
END_COM_MAP()

BEGIN_PROP_MAP(CRegistryObject)
	PROP_DATA_ENTRY("_cx", m_sizeExtent.cx, VT_UI4)
	PROP_DATA_ENTRY("_cy", m_sizeExtent.cy, VT_UI4)
	// Example entries
	// PROP_ENTRY("Property Description", dispid, clsid)
	// PROP_PAGE(CLSID_StockColorPage)
END_PROP_MAP()

BEGIN_MSG_MAP(CRegistryObject)
	CHAIN_MSG_MAP(CComControl<CRegistryObject>)
	DEFAULT_REFLECTION_HANDLER()
END_MSG_MAP()
// Handler prototypes:
//  LRESULT MessageHandler(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled);
//  LRESULT CommandHandler(WORD wNotifyCode, WORD wID, HWND hWndCtl, BOOL& bHandled);
//  LRESULT NotifyHandler(int idCtrl, LPNMHDR pnmh, BOOL& bHandled);



// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid)
	{
		static const IID* arr[] = 
		{
			&IID_IAccessRegistry,
		};
		for (int i=0; i<sizeof(arr)/sizeof(arr[0]); i++)
		{
			if (::InlineIsEqualGUID(*arr[i], riid))
				return S_OK;
		}
		return S_FALSE;
	}

// IViewObjectEx
	DECLARE_VIEW_STATUS(0)

// IAccessRegistry
public:
	STDMETHOD(DeleteValue)(/*[in]*/ BSTR bstrValueName);
	STDMETHOD(DeleteSubkey)(/*[in]*/ BSTR bstrSubKey, /*[in]*/ VARIANT_BOOL bDeleteChildKeys);
	STDMETHOD(CreateKey)(/*[in]*/ Hives hvHive, /*[in]*/ BSTR bstrKey);
	STDMETHOD(SetValue)(/*[in]*/ BSTR bstrValueName, /*[in]*/ VARIANT varValueData);
	STDMETHOD(QueryValue)(/*[in]*/ BSTR bstrValueName, /*[out]*/ VARIANT* pvarValueData);
	STDMETHOD(OpenKey)(/*[in]*/ Hives hvHive, /*[in]*/ BSTR bstrKey, /*[in, optional]*/ VARIANT varAccess);

	HRESULT OnDraw(ATL_DRAWINFO& di)
	{
		// If in design mode, place some text over the
		// control
		BOOL bRunMode = FALSE;
		GetAmbientUserMode(bRunMode);
		if ( !bRunMode ) {
			// Draw bounding rectangle
			HPEN hPen = ::CreatePen(PS_DASH,1,RGB(0,0,0));
			HBRUSH hBrush = ::CreateSolidBrush(RGB(255,0,128));
			HPEN hOldPen = (HPEN)::SelectObject(di.hdcDraw,hPen);
			HBRUSH hOldBrush = (HBRUSH)::SelectObject(di.hdcDraw,hBrush);
			RECT rcBounds = *(RECT*)di.prcBounds;
			::RoundRect(di.hdcDraw,
				rcBounds.left,
				rcBounds.top,
				rcBounds.right,
				rcBounds.bottom,
				(rcBounds.right - rcBounds.left) / 7,
				(rcBounds.bottom - rcBounds.top) / 7);

			// In design mode...pull container's ambient
			// display name and show that.
			TCHAR strName[32] = {0};
			CComBSTR bstrName;
			GetAmbientDisplayName(bstrName.m_str);
			USES_CONVERSION;
			LPTSTR strAmbientName = W2T(bstrName);

			// If there is no ambient display name, use
			// the project name.
			if ( !_tcslen(strAmbientName) ) {
				// No ambient display name
				TCHAR strProjName[32]  = {0};
				LoadString(_Module.GetResourceInstance(),IDS_PROJNAME,strName,32);
			} // if
			else {
				// Use the ambient name
				_tcsncpy(strName,strAmbientName,32);
			} // else

			// Use default font/color, place in
			// upper-left corner
			::SetTextColor(di.hdcDraw,RGB(0,0,0));
			::SetBkMode(di.hdcDraw,TRANSPARENT);
			::DrawText(di.hdcDraw,
						strName,
						_tcslen(strName),
						&rcBounds,
						DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_WORD_ELLIPSIS);

			// Clean up
			::SelectObject(di.hdcDraw,hOldPen);
			::SelectObject(di.hdcDraw,hOldBrush);
		} // if

		return S_OK;
	}

	// Helpers
protected:
	void ReportError(HRESULT hrErr)
	{
		// Format the message
		LPVOID lpMsgBuf;
		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | 
						FORMAT_MESSAGE_FROM_SYSTEM | 
						FORMAT_MESSAGE_IGNORE_INSERTS,
						NULL,
						hrErr,
						MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
						(LPTSTR)&lpMsgBuf,
						0,
						NULL);

		// Report it
		AtlReportError(__uuidof(RegistryObject),
						(LPTSTR)lpMsgBuf,
						__uuidof(IAccessRegistry),
						hrErr);

		// Free string memory
		LocalFree(lpMsgBuf);
	}

	// Attributes
protected:
	CRegKey m_CKey;

};

#endif //__REGISTRYOBJECT_H_
